@@ -2,22 +2,9 @@
 /*
  * Copyright 2016 Free Software Foundation, Inc.
  *
- * This file is part of GNU Radio
+ * This file is part of VOLK
  *
- * GNU Radio is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * GNU Radio is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with GNU Radio; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 /*!
@@ -244,7 +231,7 @@ static inline void volk_16ic_x2_dot_prod_16ic_u_sse2(lv_16sc_t* out,
 #ifdef LV_HAVE_AVX2
 #include <immintrin.h>
 
-static inline void volk_16ic_x2_dot_prod_16ic_u_axv2(lv_16sc_t* out,
+static inline void volk_16ic_x2_dot_prod_16ic_u_avx2(lv_16sc_t* out,
                                                      const lv_16sc_t* in_a,
                                                      const lv_16sc_t* in_b,
                                                      unsigned int num_points)
@@ -364,7 +351,6 @@ static inline void volk_16ic_x2_dot_prod_16ic_u_axv2(lv_16sc_t* out,
 
         _mm256_storeu_si256((__m256i*)dotProductVector,
                             result); // Store the results back into the dot product vector
-        _mm256_zeroupper();
 
         for (number = 0; number < 8; ++number) {
             dotProduct = lv_cmake(
@@ -387,7 +373,7 @@ static inline void volk_16ic_x2_dot_prod_16ic_u_axv2(lv_16sc_t* out,
 #ifdef LV_HAVE_AVX2
 #include <immintrin.h>
 
-static inline void volk_16ic_x2_dot_prod_16ic_a_axv2(lv_16sc_t* out,
+static inline void volk_16ic_x2_dot_prod_16ic_a_avx2(lv_16sc_t* out,
                                                      const lv_16sc_t* in_a,
                                                      const lv_16sc_t* in_b,
                                                      unsigned int num_points)
@@ -507,7 +493,6 @@ static inline void volk_16ic_x2_dot_prod_16ic_a_axv2(lv_16sc_t* out,
 
         _mm256_store_si256((__m256i*)dotProductVector,
                            result); // Store the results back into the dot product vector
-        _mm256_zeroupper();
 
         for (number = 0; number < 8; ++number) {
             dotProduct = lv_cmake(
