@@ -2,22 +2,9 @@
 /*
  * Copyright 2012, 2013, 2014 Free Software Foundation, Inc.
  *
- * This file is part of GNU Radio
+ * This file is part of VOLK
  *
- * GNU Radio is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * GNU Radio is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with GNU Radio; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 /*!
@@ -438,7 +425,7 @@ static inline void volk_32fc_s32fc_x2_rotator_32fc_a_avx(lv_32fc_t* outVector,
 {
     lv_32fc_t* cPtr = outVector;
     const lv_32fc_t* aPtr = inVector;
-    lv_32fc_t incr = lv_cmake(1.0, 0.0);
+    lv_32fc_t incr = lv_cmake(1.0f, 0.0f);
     lv_32fc_t phase_Ptr[4] = { (*phase), (*phase), (*phase), (*phase) };
 
     unsigned int i, j = 0;
@@ -512,7 +499,7 @@ static inline void volk_32fc_s32fc_x2_rotator_32fc_u_avx(lv_32fc_t* outVector,
 {
     lv_32fc_t* cPtr = outVector;
     const lv_32fc_t* aPtr = inVector;
-    lv_32fc_t incr = lv_cmake(1.0, 0.0);
+    lv_32fc_t incr = lv_cmake(1.0f, 0.0f);
     lv_32fc_t phase_Ptr[4] = { (*phase), (*phase), (*phase), (*phase) };
 
     unsigned int i, j = 0;
